@@ -4,9 +4,14 @@ import pandas as pd
 import torch
 import matplotlib.pyplot as plt
 
-from Fau_tools import Fau
+from Fau_tools import utility
 
 
+# ------------------------------------------------------------
+# --------------- Hyper parameters
+# ------------------------------------------------------------
+
+SAVE_NOTICE = True  # control the notice that file saved successfully.
 
 
 
@@ -49,16 +54,34 @@ class ModelManager:
 
 		Returns: None
 		"""
-		postfix = self.get_postfix()  # 87.65%  ->  8765
-		file_name = rf"{file_name}_{postfix}"
-		if only_param: torch.save(self.model.state_dict(), rf"{file_name}.pth")
-		else: torch.save(self.model, rf"{file_name}.pth")
+
+		file_name = f"{file_name}.pth"
+		if only_param: torch.save(self.model.state_dict(), rf"{file_name}")
+		else: torch.save(self.model, rf"{file_name}")
+		if SAVE_NOTICE: print(rf"{__class__.__name__}: save a model named {file_name} successfully!")
+
+	@staticmethod
+	def load(model, file_path, DEVICE=None):
+		"""
+		load the trained model that saved only parameters.
+
+		A new feature add in version 1.0.0  [test]
+
+		Args:
+			model (): load to which one
+			file_path (): the trained model path
+			DEVICE (): cpu or cuda; if it's None, it will judge for itself.
+
+		Returns: None
+		"""
+
+		if DEVICE is None: DEVICE = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
+		model.load_state_dict(torch.load(file_path))
+		model.to(DEVICE)
+
+	def get_postfix(self): return f"{round(self.accuracy * 10000)}"  # 87.65%  ->  8765
 
 
-	def get_postfix(self): return f"{round(self.accuracy * 10000)}"
-	# save and load model
-	# torch.save(model.state_dict(), "cnn_model.pth")
-	# model = model.load_state_dict(torch.load("cnn_model.pth"))
 
 
 
@@ -87,13 +110,16 @@ class TrainRecorder:
 
 		Returns: None
 		"""
-		with open(rf"{file_name}.csv", "w") as file:
+		file_name = rf"{file_name}.csv"
+		with open(rf"{file_name}", "w") as file:
 			col_list = ", ".join(("loss", "accuracy")) + "\n"
 			file.write(col_list)
 			for loss, accuracy in zip(self.loss_list, self.accuracy_list):
 				line = f"{loss:.4f}, {accuracy:.4f}\n"
 				file.write(line)
 
+		if SAVE_NOTICE: print(rf"{__class__.__name__}: save a record file named {file_name} successfully!")
+
 
 
 class TimeManager:
@@ -123,11 +149,12 @@ class TimeManager:
 
 
 
+
 # ------------------------------------------------------------
 # --------------- Function --- training
 # ------------------------------------------------------------
 
-def show_progress(now, total, loss=None, accuracy=None, time_manager=None):
+def _show_progress(now, total, loss=None, accuracy=None, time_manager=None):
 	"""
 	A function that displays a progress bar.
 
@@ -154,8 +181,8 @@ def show_progress(now, total, loss=None, accuracy=None, time_manager=None):
 		average_time, elapsed_time = time_manager.get_average_time(), time_manager.get_elapsed_time()
 		total_time = total * average_time
 
-		elapsed_time = Fau.time_to_human(elapsed_time)
-		total_time = Fau.time_to_human(total_time)
+		elapsed_time = utility.time_to_human(elapsed_time)
+		total_time = utility.time_to_human(total_time)
 
 		show += f" [{elapsed_time}<{total_time}]"
 
@@ -181,21 +208,22 @@ def calc_accuracy(model, test_loader, DEVICE=None):
 
 	model.eval()  # no dropout...
 
-	test_result = list()  # for calculating the average accuracy rate.
-	for (test_x, test_y) in test_loader:
-		test_x, test_y = test_x.to(DEVICE), test_y.to(DEVICE)
-		test_output: torch.Tensor = model(test_x)
-		test_prediction: torch.Tensor = test_output.argmax(1)  # get classification result set
-		cur_accuracy: torch.Tensor = sum(test_prediction.eq(test_y)) / test_y.size(0)
-		test_result.append(cur_accuracy.item())  # tensor -> scaler
+	with torch.no_grad():
+		test_result = list()  # for calculating the average accuracy rate.
+		for (test_x, test_y) in test_loader:
+			test_x, test_y = test_x.to(DEVICE), test_y.to(DEVICE)
+			test_output: torch.Tensor = model(test_x)
+			test_prediction: torch.Tensor = test_output.argmax(1)  # get classification result set
+			cur_accuracy: torch.Tensor = sum(test_prediction.eq(test_y)) / test_y.size(0)
+			test_result.append(cur_accuracy.item())  # tensor -> scaler
+		accuracy: float = sum(test_result) / len(test_result)  # get average accuracy
 
 	model.train()  # recover
-	accuracy: float = sum(test_result) / len(test_result)  # get average
 	return round(accuracy, 6)
 
 
 
-@Fau.calc_time
+@utility.calc_time
 def torch_train(model, train_loader, test_loader, optimizer, loss_function, EPOCH=100, name=None, save_parameters=False, DEVICE=None):
 	"""
 	A function for training the best model.
@@ -208,7 +236,7 @@ def torch_train(model, train_loader, test_loader, optimizer, loss_function, EPOC
 		loss_function (): loss function
 		EPOCH (): total EPOCH
 		name (): if the model need to be saved, please pass the model name without postfix.
-		save_parameters (): whether the optimizer and the loss function need to be saved.
+		save_parameters (): whether the optimizer and the loss function need to be saved. name () is must.
 		DEVICE (): cpu or cuda; if it's None, it will judge for itself.
 
 	Returns: None
@@ -235,6 +263,8 @@ def torch_train(model, train_loader, test_loader, optimizer, loss_function, EPOC
 		finally:
 			print(f"{'='*10} Training in {DEVICE_NAME} {'='*10}")
 
+	if train_loader.batch_size == 1:
+		print("Warning: you shouldn't set the batch_size to 1. since if the NN uses BN, it will arise an error.")
 
 	# for saving training data
 	model_manager = ModelManager()
@@ -257,10 +287,11 @@ def torch_train(model, train_loader, test_loader, optimizer, loss_function, EPOC
 			optimizer.step()
 
 		# end of epoch
-		time_manager.time_tick()
 		# noinspection PyUnboundLocalVariable
-		loss_value, accuracy = loss.item(), calc_accuracy(model, test_loader)  # get loss and acc
-		show_progress(epoch, EPOCH, loss_value, accuracy, time_manager)
+		loss_value, accuracy = loss.item(), calc_accuracy(model, test_loader, DEVICE)  # get loss and acc
+		time_manager.time_tick()  # tick current time
+		_show_progress(epoch, EPOCH, loss_value, accuracy, time_manager)
+
 		# update and record
 		model_manager.update(model, loss_value, accuracy)
 		train_recorder.update(loss_value, accuracy)
@@ -269,15 +300,22 @@ def torch_train(model, train_loader, test_loader, optimizer, loss_function, EPOC
 	if name is None: return  # no save
 
 	# save model and process
-	model_manager.save(name)
-	train_recorder.save(f"{name}_{model_manager.get_postfix()}")
+	SAVE_NAME = f"{name}_{model_manager.get_postfix()}"
+	model_manager.save(SAVE_NAME)
+	train_recorder.save(SAVE_NAME)
+
 
 	if save_parameters:  # save the parameters
-		parameters_filename = f"{name}_{model_manager.get_postfix()}.txt"
+		parameters_filename = f"{SAVE_NAME}.txt"
 		with open(parameters_filename, "w") as file:
 			file.write(f"optimizer: \n{str(optimizer)}\n")
 			file.write(f"{'-' * 20}\n")
-			file.write(f"loss function: \n{str(loss_function)}")
+			file.write(f"loss function: \n{str(loss_function)}\n")
+			file.write(f"{'-' * 20}\n")
+			file.write(f"batch size: {train_loader.batch_size}\n")
+		if SAVE_NOTICE: print(f"{torch_train.__name__}: save a parameter file named {parameters_filename} successfully!")
+
+
 
 
 
@@ -336,4 +374,4 @@ def draw_plot(*args, legend_names=None, x_name=None, y_name=None, percent=False)
 		y_ticks_percent = [f"{op:.2%}" for op in y_ticks]  # convert to percent
 		plt.yticks(y_ticks, y_ticks_percent)
 
-	# plt.show()  # Note: This will lead to show the figure one by one.
+# plt.show()  # Note: This will lead to show the figure one by one.
